﻿B4R=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=2.51
@EndOfDesignText@
Sub Process_Globals
  Dim fftRe(64) As Double ' real imag data, magnitude sqr
  Dim fftIm(64) As Double
  Dim fftMag(64 / 2) As Double

  Dim fftCos(64 / 2) As Double ' Sin Cos coefficients, Blackman window
  Dim fftSin(64 / 2) As Double
  Dim BlkWindow(64) As Double

  Dim fftN As Int ' FFT size
  Dim fftM As Int ' FFT stages
End Sub

' FFT Routines
' Copyright 2006-2007 Columbia University.
' This file is part of MEAPsoft.
' MEAPsoft is free software you can redistribute it And/Or modify
' it under the terms of the GNU General Public License version 2 As
' published by the Free Software Foundation.

Sub InitFFT(n As Int, m As Int) As Boolean
  fftN = n
  fftM = m

  If (n <> (Bit.ShiftLeft(1, m))) Then Return(False) ' Make sure n is a Power of 2
    
  For i=0 To n/2 - 1
    fftCos(i) = Cos(-Main.TwoPi * i / n)
    fftSin(i) = Sin(-Main.TwoPi * i / n)
  Next

  ' Make a blackman window: w(n)=0.42-0.5cos{(2*PI*n)/(N-1)}+0.08cos{(4*PI*n)/(N-1)}
        
  For i=0 To n - 1
    BlkWindow(i) = 0.42 - 0.5 * Cos(Main.TwoPi * i / (fftN-1)) + 0.08 * Cos(2 * Main.TwoPi * i / (fftN-1))
  Next
    
  Return(True)
End Sub
  
Sub CopyFFTData(data() As Int, dReal() As Double, dImag() As Double, n As Int, bias As Double) ' Copy, apply Blackman, zero imag
  For i=0 To n - 1
    dReal(i) = data(i) * BlkWindow(i) + bias
    dImag(i) = 0.0
  Next
End Sub

Sub ScaleFFT(x() As Double, y() As Double, scl As Double, n As Int) ' Calc magnitude square root
  For i=0 To n - 1
    x(i) = x(i) * scl
    y(i) = x(i) * scl
  Next
End Sub
    
Sub CalcFFTMag(x() As Double, y() As Double, srt As Int, n As Int) ' Calc magnitude square root
  For i=srt To (srt + n) - 1
    fftMag(i) = Sqrt(x(i) * x(i) + y(i) * y(i))
  Next
End Sub
    
' from: gusano79
' https:'www.daniweb.com/programming/software-development/threads/354944/creating-a-sine-wave-at-a-specific-frequency-and-sample-rate
'
' A sample rate of 44.1 kHz is 44,100 samples/second
' A 1 kHz wave is 1,000 cycles/second
' The sine function has a cycle length of 2*pi
'
' So. You need to scale your sine function such that 1,000 sine wave cycles would fit into 44,100 samples.
' Here's a rough look at how you'd derive code to fill the buffer:
'  
' for(i = 0 i < 10000 i++) {
'    i is the sample index
'    Straight sine function means one cycle every 2*pi samples:
'    buffer(i) = sin(i) 
'    Multiply by 2*pi--now it's one cycle per sample:
'    buffer(i) = sin((2 * pi) * i) 
'    Multiply by 1,000 samples per second--now it's 1,000 cycles per second:
'    buffer(i) = sin(1000 * (2 * pi) * i)
'    Divide by 44,100 samples per second--now it's 1,000 cycles per 44,100
'    samples, which is just what we needed:
'
'    buffer(i) = sin(1000 * (2 * pi) * i / 44100)
' }
'  
'  public TestFFT(double amp, double fr) { ' with amp and frequency sampled at sample rate
'    InitFFT(fftN)
'
'    for(int i = 0 i < fftN i++) {
'      fftRe(i) = amp * Math.cos(fr * (2 * Math.PI) * i / sampleRate)
'      fftIm(i) = 0
'    }
'        
'    CalcFFT(fftRe, fftIm)
'    ScaleFFT(fftRe, fftIm, 1.0 / fftN, fftN)
'    CalcFFTMag(fftRe, fftIm, 0, fftN / 2)
'  }

' fft.c
' Douglas L. Jones 
' University of Illinois at Urbana-Champaign 
' January 19, 1992 
' http:'cnx.rice.edu/content/m12016/latest/
' 
' fft: in-place radix-2 DIT DFT of a complex input 
' 
' input: 
'   n: length of FFT: must be a power of two 
'   m: n = 2**m 
' input/output 
'   x: double array of length n with real part of data 
'   y: double array of length n with imag part of data 
' 
'   Permission to copy and use this program is granted 
'   as long as this header is included. 

Sub CalcFFT(x() As Double, y() As Double)
  Dim i, j, k, n1, n2, a As Int
  Dim c, s, t1, t2 As Double

  ' Bit-reverse
  j = 0
  n2 = fftN / 2
     
  For i=1 To fftN - 2
    n1 = n2
 
    Do While (j >= n1)
      j = j - n1
      n1 = n1 / 2
    Loop

    j = j + n1
        
    If (i < j) Then
      t1 = x(i)
      x(i) = x(j)
      x(j) = t1
      t1 = y(i)
      y(i) = y(j)
      y(j) = t1
    End If
  Next

  ' FFT
  n1 = 0
  n2 = 1
      
  For i = 0 To fftM - 1
    n1 = n2
    n2 = n2 + n2
    a = 0
        
    For j = 0 To n1 - 1
      c = fftCos(a)
      s = fftSin(a)
      a = a + Bit.ShiftLeft(1, fftM - i - 1)

      For k = j To fftN - 1 Step n2
        t1 = c * x(k+n1) - s * y(k+n1)
        t2 = s * x(k+n1) + c * y(k+n1)
        x(k+n1) = x(k) - t1
        y(k+n1) = y(k) - t2
        x(k) = x(k) + t1
        y(k) = y(k) + t2
      Next
    Next
  Next
End Sub
